/*
IMG unpack plugin
Copyright (C) 2002 IvGzury
	ivgzury@hotmail.com

changes for FAT12/FAT16 drive images by Holger Klabunde
 hk@holger-klabunde.de
 http://www.holger-klabunde.de

This program is absolutely free software.
If you have any remarks or problems, please don't
hesitate to send me an email.

It was made for Windows Commander.
Windows Commander: is an excellent file manager 
                   made by Christian Ghisler
*/

#include "stdafx.h"
#include "wcxhead.h"

#define FAT12	12
#define FAT16	16
#define FAT32	32

#define BUFFSIZE 512*2048

//partitiontable offsets
#define PART0_TABLE_OFFSET  0x01BE
#define PART1_TABLE_OFFSET  PART0_TABLE_OFFSET+16
#define PART2_TABLE_OFFSET  PART0_TABLE_OFFSET+32
#define PART3_TABLE_OFFSET  PART0_TABLE_OFFSET+48

//force visual c++ v5.00 to use byte boundaries for
//all data in a structure, so we can replace the ugly
//BYTE arrays which are difficult to handle
#pragma pack( push) //store old data memory alignment
#pragma pack( 1)    //use byte boundaries for structures

struct endboot12 //FAT12/16 defs beginning at offset 36
 {
	unsigned char  BS_DrvNum;
	unsigned char  BS_Reserved1;
	unsigned char  BS_BootSig;
	unsigned char  BS_VolID[4];
	char           BS_VolLab[11];
	char           BS_FilSysType[8];
	unsigned char  remaining_part[450];
 };

struct endboot32 //FAT32 defs beginning at offset 36
  {
	unsigned long  BPB_FATSz32; //4 bytes
	unsigned short BPB_ExtFlags; //2 bytes
	unsigned short BPB_FSVer; //2 bytes
	unsigned long  BPB_RootClus; //4 bytes
	unsigned short BPB_FSInfo; //2 bytes
	unsigned short BPB_BkBootSec; //2 bytes
	unsigned char  BPB_Reserved[12];
	unsigned char  BS_DrvNum;
	unsigned char  BS_Reserved1;
	unsigned char  BS_BootSig;
	unsigned long  BS_VolID; //4 bytes
	char           BS_VolLab[11];
	char           BS_FilSysType[8];
	unsigned char  remaining_part[422];
}; 

union endboot
 {
   struct endboot12 eb12; //now both share the same memory
   struct endboot32 eb32;
 };

struct tFAT12BootSec 
{
	unsigned char  BS_jmpBoot[3];
	char           BS_OEMName[8];
	unsigned short BPB_BytesPerSec; //2 bytes
	unsigned char  BPB_SecPerClus;
	unsigned short BPB_RsvdSecCnt; //2 bytes
	unsigned char  BPB_NumFATs;
	unsigned short BPB_RootEntCnt; //2 bytes
	unsigned short BPB_TotSec16; //2 bytes
	unsigned char  BPB_Media;
	unsigned short BPB_FATSz16; //2 bytes
	unsigned short BPB_SecPerTrk; //2 bytes
	unsigned short BPB_NumHeads; //2 bytes
	unsigned long  BPB_HiddSec; //4 bytes
	unsigned long  BPB_TotSec32; //4 bytes

    endboot eb;
};
#pragma pack( pop) //restore old data alignment

#pragma pack( push) //store old data alignment
#pragma pack( 1)    //use byte boundaries for structures
typedef struct
{
	unsigned char data[BUFFSIZE];
} tFAT12Table;
#pragma pack( pop) //restore old data alignment

#pragma pack( push) //store old data memory alignment
#pragma pack( 1)    //use byte boundaries for structures
typedef struct
{
	char           DIR_Name[11];
	unsigned char  DIR_Attr;
	unsigned char  DIR_NTRes;
	unsigned char  DIR_CrtTimeTenth;
	unsigned short DIR_CrtTime;
	unsigned short DIR_CrtDate;
	unsigned short DIR_LstAccDate;
	unsigned short DIR_FstClusHI;
	unsigned short DIR_WrtTime;
	unsigned short DIR_WrtDate;
	unsigned short DIR_FstClusLO;
	unsigned long  DIR_FileSize;
} tFAT12DirEntry;
#pragma pack( pop) //restore old data alignment

typedef struct 
{
//	tFAT12DirEntry data[512/32];
	tFAT12DirEntry data[BUFFSIZE];
} tDirSector;

#pragma pack( push) //store old data memory alignment
#pragma pack( 1)    //use byte boundaries for structures
typedef struct 
{
	unsigned char data[BUFFSIZE];
} tCluster;
#pragma pack( pop) //restore old data alignment

//--------End of  FAT Definitions-------------

//----------------IMG Definitions-------------

typedef struct tDirEntry
{
	char FileName[260];
	char PathName[260];
	unsigned long FileSize;
	unsigned long FileTime;
	unsigned char FileAttr;
	unsigned long FirstClus;
	tDirEntry* next;
	tDirEntry* prev;
} tDirEntry;

typedef struct
{ 
	char archname[MAX_PATH]; 
	HANDLE hArchFile;    //opened file handle

	tFAT12Table *fattable;
	tDirEntry *entrylist;

	unsigned long rootarea;    //sector offset to rootDir area
	unsigned long dataarea;    //sector offset to data area
	unsigned long rootentcnt; 
	unsigned long fatsize;     //sectors for ONE FAT
	unsigned long counter;

	tChangeVolProc pLocChangeVol;
	tProcessDataProc pLocProcessData;
} tArchive;

typedef tArchive *myHANDLE;

#define AT_OK      0
#define AT_VOL     1
#define AT_DIR     2
#define AT_LONG    3
#define AT_INVALID 4

#define ATTR_READONLY     0x01
#define ATTR_HIDDEN       0x02
#define ATTR_SYSTEM       0x04
#define ATTR_VOLUME_ID    0x08
#define ATTR_DIRECTORY    0x10
#define ATTR_ARCHIVE      0x20
#define ATTR_LONG_NAME    0x0F

#define DN_OK      0
#define DN_DELETED 1
#define DN_FREE    2 // indicates that all the others are free too
#define DN_UNKNOWN 3

//Function prototypes
unsigned int MakeUINT(unsigned char *dat);
unsigned long MakeULONG(unsigned char *dat);
int DIR_AttrToFileAttr(unsigned char DIR_Attr,unsigned char *FileAttr);
int ValidChar(char mychar);
int DIR_NameToFileName(char *DIR_Name,char *FileName);
int IMG_NextItem(myHANDLE hArcData, tHeaderData *HeaderData) ;
